/*
 *    This file was created to modify the slopeXXX.full.cub files to 
 *    flat cubes flatXXX.YYYY.cub, where XXX is the file number and YYYY is the 
 *    instrument mode ID.  Before using this program, the *.img and *.lbl files 
 *    should be imported using vicar2isis or pds2isis and dealing with any VAX 
 *    formatted images in isis2.  
 * 
 *    This program requires that the slopeXXX.full.cub files are in the parent 
 *    slope/ directory and will create the final files in the flat/ subdirectory 
 *    of the slope/ parent directory.  Other files created by this program will 
 *    be of the form slopeXXX.operationName.YYYY.cub where XXX is the filenumber, 
 *    operationName is the name of the operation performed during this step, and 
 *    YYYY is the instrument mode ID for the image. Since these cubes are not 
 *    needed by the cisscal application, they will be created in the slope2flat/ 
 *    subdirectory so that they may easily be deleted after this program is run.
 */

#include "Isis.h"
#include "Cube.h"
#include "FileList.h"
#include "Filename.h"
#include "iException.h"
#include "UserInterface.h"
#include "Pvl.h"
#include "iString.h"


using namespace std; 
using namespace Isis;


void IsisMain() {

// Get the list of cubes

  UserInterface &ui = Application::GetUserInterface();
  FileList inList(ui.GetFilename("FROMLIST"));
  iString num;
  int pos;
  string parameters = "";
  string pvlOut = "";
  PvlGroup statsResults;
  iString avg = "";
  for (int i = 0; i < (int)inList.size(); i++) {
    pos = inList[i].find("e");
    num = inList[i].substr(pos+1);

    try{
      // CREATE FLAT FILES FOR IMAGES WITH SUMMING MODE = 1 (FULL) BY FOLLOWING THE IDL CISSCAL'S ALGORITHM:
      // --find reciprocal of image
      // --create an image from the center 400 x 400 pixels
      // --find average of the center
      // --divide the reciprocal by this center average
      parameters = "f1=../" + inList[i] + ".full.cub" + 
                        " to=" + inList[i] + ".full.reciprocal.cub" +
                        " equation=1/f1";
      Isis::iApp->Exec("fx",parameters);
      parameters = "from=" + inList[i] + ".full.reciprocal.cub" + 
                        " to=" + inList[i] + ".full.center.cub" +
                        " sample=313 nsamples=400 line=313 nlines=400";
      Isis::iApp->Exec("crop",parameters);
      pvlOut = "stats" + num + ".full.pvl";
      parameters = "from=" + inList[i] + ".full.center.cub" + 
                          " to=" + pvlOut;
      Isis::iApp->Exec("stats",parameters);
      Pvl pFull;
      pFull.Read(pvlOut);
      statsResults = pFull.FindGroup("Results",Pvl::Traverse);
      avg = ((string) statsResults["Median"]);
      parameters = "f1=" + inList[i] + ".full.reciprocal.cub" + 
                   " to=../flat/flat" + num + ".full.cub" + 
                   " equation=f1/" + avg;
      Isis::iApp->Exec("fx",parameters);
  
      // CREATE FLAT FILES FOR IMAGES WITH SUM2 BY FOLLOWING THE IDL CISSCAL'S ALGORITHM:
      // --reduce slope image to sum2 (512 by 512)
      // --find reciprocal of summed image
      // --enlarge reciprocal back out to 1024 by 1024
      // --create an image from the center 400 x 400 pixels
      // --find average of the center
      // --divide the enlarged reciprocal by this center average
      // --reduce back down to 512 x 512
      //*********************************************************************************
      // --reduce slope image to sum2 (512 by 512)
      parameters = "from=../" + inList[i] + ".full.cub" + 
                        " to=" + inList[i] + ".sum2.cub" +
                        " onl=512 ons=512";
      Isis::iApp->Exec("reduce",parameters);
      // --find reciprocal of summed image
      parameters = "f1=" + inList[i] + ".sum2.cub" + 
                        " to=" + inList[i] + ".sum2.reciprocal.cub" +
                        " equation=1/f1";
      Isis::iApp->Exec("fx",parameters);
      // --enlarge reciprocal back out to 1024 by 1024
      parameters = "from=" + inList[i] + ".sum2.reciprocal.cub" + 
                        " to=" + inList[i] + ".sum2.enlargedRecip.cub" +
                        " sscale=2 lscale=2 interp=bilinear";
      Isis::iApp->Exec("enlarge",parameters);
      // --create an image from the center 400 x 400 pixels
      parameters = "from=" + inList[i] + ".sum2.enlargedRecip.cub" + 
                        " to=" + inList[i] + ".sum2.center.cub" +
                        " sample=313 nsamples=400 line=313 nlines=400";
      Isis::iApp->Exec("crop",parameters);
      // --find average of the center
      pvlOut = "stats" + num + ".sum2.pvl";
      parameters = "from=" + inList[i] + ".sum2.center.cub" + 
                          " to=" + pvlOut;
      Isis::iApp->Exec("stats",parameters);
      // --divide the enlarged reciprocal by this center average
      Pvl pSum2;
      pSum2.Read(pvlOut);
      statsResults = pSum2.FindGroup("Results",Pvl::Traverse);
      avg = ((double) statsResults["Median"]);
      parameters = "f1=" + inList[i] + ".sum2.enlargedRecip.cub" + 
                   " to=" + inList[i] + ".sum2.divided.cub" + 
                   " equation=f1/" + avg;
      Isis::iApp->Exec("fx",parameters);
      // --reduce back down to 512 x 512
      parameters = "from=" + inList[i] + ".sum2.divided.cub" + 
                        " to=../flat/flat" + num + ".sum2.cub" +
                        " onl=512 ons=512";
      Isis::iApp->Exec("reduce",parameters);
  

      // CREATE FLAT FILES FOR IMAGES WITH SUM4 BY FOLLOWING THE IDL CISSCAL'S ALGORITHM:
      // --reduce slope image to sum4 (256 by 526)
      // --find reciprocal of summed image
      // --enlarge reciprocal back out to 1024 by 1024
      // --create an image from the center 400 x 400 pixels
      // --find average of the center
      // --divide the enlarged reciprocal by this center average
      // --reduce back down to 256 x 256
      parameters = "from=../" + inList[i] + ".full.cub" + 
                        " to=" + inList[i] + ".sum4.cub" +
                        " onl=256 ons=256";
      Isis::iApp->Exec("reduce",parameters);
      parameters = "f1=" + inList[i] + ".sum4.cub" + 
                        " to=" + inList[i] + ".sum4.reciprocal.cub" +
                        " equation=1/f1";
      Isis::iApp->Exec("fx",parameters);
      parameters = "from=" + inList[i] + ".sum4.reciprocal.cub" + 
                        " to=" + inList[i] + ".sum4.enlargedRecip.cub" +
                        " sscale=4 lscale=4 interp=bilinear";
      Isis::iApp->Exec("enlarge",parameters);
      parameters = "from=" + inList[i] + ".sum4.enlargedRecip.cub" + 
                        " to=" + inList[i] + ".sum4.center.cub" +
                        " sample=313 nsamples=400 line=313 nlines=400";
      Isis::iApp->Exec("crop",parameters);
      pvlOut = "stats" + num + ".sum4.pvl";
      parameters = "from=" + inList[i] + ".sum4.center.cub" + 
                          " to=" + pvlOut;
      Isis::iApp->Exec("stats",parameters);
      Pvl pSum4;
      pSum4.Read(pvlOut);
      statsResults = pSum4.FindGroup("Results",Pvl::Traverse);
      avg = ((double) statsResults["Median"]);
      parameters = "f1=" + inList[i] + ".sum4.enlargedRecip.cub" + 
                   " to=" + inList[i] + ".sum4.divided.cub" + 
                   " equation=f1/" + avg;
      Isis::iApp->Exec("fx",parameters);
      parameters = "from=" + inList[i] + ".sum4.divided.cub" + 
                        " to=../flat/flat" + num + ".sum4.cub" +
                        " onl=256 ons=256";
      Isis::iApp->Exec("reduce",parameters);
    }
    catch (iException &e3){
      e3.Report();
    }
  }
} // end of isis main

